@@ -1,5 +1,5 @@
 /*
-    $Id: vcd-info.c,v 1.1.2.3 2003/05/18 02:37:10 rocky Exp $
+    $Id: vcd-info.c,v 1.1.2.19 2003/10/03 09:11:35 rocky Exp $
 
     Copyright (C) 2001,2002 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002,2003 Rocky Bernstein <rocky@panix.com>
@@ -40,17 +40,21 @@
 
 #include <cdio/cdio.h>
 #include <cdio/logging.h>
+#include <cdio/iso9660.h>
 
 /* Eventually move above libvcd includes but having vcdinfo including. */
 #include <libvcd/info.h>
-#include <libvcd/info_private.h>
+#include <libvcd/files.h>
+
+/* FIXME */
+#include <libvcd/files_private.h>
 
 /* Private headers */
 #include "bitvec.h"
+#include "bytesex.h"
 #include "pbc.h"
-#include "image_fs.h"
 
-static const char _rcsid[] = "$Id: vcd-info.c,v 1.1.2.3 2003/05/18 02:37:10 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd-info.c,v 1.1.2.19 2003/10/03 09:11:35 rocky Exp $";
 
 static const char DELIM[] = \
 "----------------------------------------" \
@@ -59,7 +63,7 @@ static const char DELIM[] = \
 /* global static vars */
 static struct gl_t
 {
-  vcdinfo_source_t source_type;
+  int  source_type;
   char *access_mode;
 
   /* Boolean values set by command-line options to reduce output.
@@ -109,7 +113,7 @@ static struct gl_t
       int psds;  /* Show PSD size. */
       int res;   /* Show restriction */
       int seg;   /* Show first segment address */
-      int segn;  /* Show maximum segment number */
+      int segn;  /* Show number of segments */
       int segs;  /* Show segments */
       int spec;  /* Show special info */
       int start; /* Show volume start times */
@@ -191,20 +195,23 @@ typedef enum {
 static vcd2_ext_pbc_status_t
 _has_vcd2_ext_pbc (const vcdinfo_obj_t *obj)
 {
-  vcd_image_stat_t statbuf;
+  iso9660_stat_t statbuf;
+  CdIo *img;
+  
   if (!vcdinfo_has_pbc(obj))
     return PBC_VCD2_NO_PBC;
 
-  if (obj->vcd_type != VCD_TYPE_VCD2)
+  if (vcdinfo_get_VCD_type(obj) != VCD_TYPE_VCD2)
     return PBC_VCD2_NOPE;
-  
-  if (vcd_image_source_fs_stat (obj->img, "EXT/LOT_X.VCD;1", &statbuf))
+
+  img = vcdinfo_get_cd_image(obj);
+  if (iso9660_fs_stat (img, "EXT/LOT_X.VCD;1", &statbuf, true))
     return PBC_VCD2_NO_LOT_X;
 
   if (statbuf.size != ISO_BLOCKSIZE * LOT_VCD_SIZE)
     return PBC_VCD2_BAD_LOT_SIZE;
 
-  if (!vcd_image_source_fs_stat (obj->img, "EXT/PSD_X.VCD;1", &statbuf))
+  if (!iso9660_fs_stat (img, "EXT/PSD_X.VCD;1", &statbuf, true))
     return PBC_VCD2_EXT;
   else 
     return PBC_VCD2_NO_PSD_X;
@@ -213,16 +220,16 @@ _has_vcd2_ext_pbc (const vcdinfo_obj_t *obj)
 static void
 dump_lot (const vcdinfo_obj_t *obj, bool ext)
 {
-  const LotVcd *lot = ext ? obj->lot_x : obj->lot;
+  const LotVcd *lot = ext ? vcdinfo_get_lot_x(obj) : vcdinfo_get_lot(obj);
   
   unsigned n, tmp;
   const uint16_t max_lid = vcdinfo_get_num_LIDs(obj);
-  unsigned mult = obj->info.offset_mult;
+  unsigned int mult = vcdinfo_get_offset_mult(obj);
 
   if (!gl.show.no.header)
     fprintf (stdout, 
-             (obj->vcd_type == VCD_TYPE_SVCD 
-              || obj->vcd_type == VCD_TYPE_HQVCD)
+             (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
              ? "SVCD/LOT.SVD\n"
              : (ext ? "EXT/LOT_X.VCD\n": "VCD/LOT.VCD\n"));
 
@@ -251,13 +258,15 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
 {
   VcdListNode *node;
   unsigned n = 0;
-  unsigned mult = obj->info.offset_mult;
-  const uint8_t *psd = ext ? obj->psd_x : obj->psd;
-  VcdList *offset_list = ext ? obj->offset_x_list : obj->offset_list;
+  unsigned int mult = vcdinfo_get_offset_mult(obj);
+  const uint8_t *psd = ext 
+    ? vcdinfo_get_psd_x(obj) : vcdinfo_get_psd(obj);
+  VcdList *offset_list = ext 
+    ? vcdinfo_get_offset_x_list(obj) : vcdinfo_get_offset_list(obj);
 
   fprintf (stdout, 
-           (obj->vcd_type == VCD_TYPE_SVCD 
-            || obj->vcd_type == VCD_TYPE_HQVCD)
+           (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+            || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
            ? "SVCD/PSD.SVD\n"
            : (ext ? "EXT/PSD_X.VCD\n": "VCD/PSD.VCD\n"));
 
@@ -277,7 +286,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
             const PsdPlayListDescriptor *pld = (const void *) (psd + _rofs);
             
             int i;
-            uint16_t lid = vcdinf_get_lid_from_pld(pld);
+            uint16_t lid = vcdinf_pld_get_lid(pld);
 
             fprintf (stdout,
                      " PSD[%.2d] (%s): play list descriptor\n"
@@ -287,16 +296,19 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      n, vcdinfo_ofs2str (obj, ofs->offset, ext),
                      pld->noi, lid, 
                      _vcd_bool_str(vcdinfo_is_rejected(uint16_from_be(pld->lid))),
-                     vcdinfo_ofs2str(obj, vcdinf_get_prev_from_pld(pld), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_next_from_pld(pld), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_return_from_pld(pld),
+                     vcdinfo_ofs2str(obj, 
+                                     vcdinf_pld_get_prev_offset(pld), ext),
+                     vcdinfo_ofs2str(obj, 
+                                     vcdinf_pld_get_next_offset(pld), ext),
+                     vcdinfo_ofs2str(obj, 
+                                     vcdinf_pld_get_return_offset(pld),
                                      ext),
-                     vcdinf_get_play_time(pld), vcdinfo_get_wait_time (pld),
-                     vcdinfo_get_autowait_time(pld));
+                     vcdinf_get_play_time(pld), vcdinf_get_wait_time (pld),
+                     vcdinf_get_autowait_time(pld));
 
             for (i = 0; i < pld->noi; i++) {
               fprintf (stdout, "  play-item[%d]: %s\n", i, 
-                       vcdinfo_pin2str(vcdinf_get_play_item_from_pld(pld,i)));
+                       vcdinfo_pin2str(vcdinf_pld_get_play_item(pld,i)));
             }
             fprintf (stdout, "\n");
           }
@@ -306,7 +318,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
           {
             const PsdEndListDescriptor *d = (const void *) (psd + _rofs);
             fprintf (stdout, " PSD[%.2d] (%s): end list descriptor\n", n, vcdinfo_ofs2str (obj, ofs->offset, ext));
-            if (obj->vcd_type != VCD_TYPE_VCD2)
+            if (vcdinfo_get_VCD_type(obj) != VCD_TYPE_VCD2)
               {
                 fprintf (stdout, "  next disc number: %d (if 0 stop PBC handling)\n", d->next_disc);
                 fprintf (stdout, "  change picture item: %s\n", vcdinfo_pin2str (uint16_from_be (d->change_pic)));
@@ -321,7 +333,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
             const PsdSelectionListDescriptor *d =
               (const void *) (psd + _rofs);
             int i;
-            const unsigned int lid=vcdinf_get_lid_from_psd(d);
+            const unsigned int lid=vcdinf_psd_get_lid(d);
 
             fprintf (stdout,
                      "  PSD[%.2d] (%s): %sselection list descriptor\n"
@@ -336,21 +348,22 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      vcdinf_get_num_selections(d),
                      vcdinf_get_bsn(d),
                      lid, 
-                     _vcd_bool_str (vcdinf_get_lid_rejected_from_psd(d)),
-                     vcdinfo_ofs2str(obj, vcdinf_get_prev_from_psd(d), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_next_from_psd(d), ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_return_from_psd(d),ext),
-                     vcdinfo_ofs2str(obj, vcdinf_get_default_from_psd(d), ext),
-                     vcdinfo_ofs2str(obj, vcdinfo_get_timeout_LID(d), ext),
-                     vcdinfo_get_timeout_time(d),
+                     _vcd_bool_str (vcdinf_psd_get_lid_rejected(d)),
+                     vcdinfo_ofs2str(obj, vcdinf_psd_get_prev_offset(d), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_psd_get_next_offset(d), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_psd_get_return_offset(d),ext),
+                     vcdinfo_ofs2str(obj, 
+                                     vcdinf_psd_get_default_offset(d),ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_timeout_offset(d), ext),
+                     vcdinf_get_timeout_time(d),
                      vcdinf_get_loop_count(d), 
                      _vcd_bool_str (vcdinf_has_jump_delay(d)),
-                     vcdinfo_pin2str (vcdinf_get_itemid_from_psd(d)));
+                     vcdinfo_pin2str (vcdinf_psd_get_itemid(d)));
 
             for (i = 0; i < vcdinf_get_num_selections(d); i++)
               fprintf (stdout, "  ofs[%d]: %s\n", i,
                        vcdinfo_ofs2str (obj, 
-                                        vcdinf_get_offset_from_psd(d, i), 
+                                        vcdinf_psd_get_offset(d, i), 
                                         ext));
 
             if (type == PSD_TYPE_EXT_SELECTION_LIST 
@@ -391,16 +404,16 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
 }
 
 static void
-dump_info (const vcdinfo_obj_t *obj)
+dump_info (vcdinfo_obj_t *obj)
 {
-  const InfoVcd *info = &obj->info;
-  unsigned int num_segments = vcdinfo_get_num_segments(obj);
+  const InfoVcd *info = vcdinfo_get_infoVcd(obj);
+  segnum_t num_segments = vcdinfo_get_num_segments(obj);
   int n;
 
   if (!gl.show.no.header)
     fprintf (stdout, 
-             (obj->vcd_type == VCD_TYPE_SVCD 
-              || obj->vcd_type == VCD_TYPE_HQVCD)
+             (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
              ? "SVCD/INFO.SVD\n" 
              : "VCD/INFO.VCD\n");
 
@@ -432,7 +445,8 @@ dump_info (const vcdinfo_obj_t *obj)
       
       fprintf (stdout, " flags:\n");
       fprintf (stdout, 
-               ((obj->vcd_type == VCD_TYPE_SVCD || obj->vcd_type == VCD_TYPE_HQVCD) 
+               ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+                 || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD) 
                 ? "  reserved1: %s\n"
                 : "  karaoke area: %s\n"),
                _vcd_bool_str (info->flags.reserved1));
@@ -450,7 +464,8 @@ dump_info (const vcdinfo_obj_t *obj)
     fprintf (stdout, "  start track #2: %s\n", _vcd_bool_str (info->flags.use_track3));
   if (gl.show.info.pbc) {
     fprintf (stdout, 
-             ((obj->vcd_type == VCD_TYPE_SVCD || obj->vcd_type == VCD_TYPE_HQVCD) 
+             ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+               || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD) 
               ? "  reserved2: %s\n"
               : "  extended pbc: %s\n"),
              _vcd_bool_str (info->flags.pbc_x));
@@ -486,49 +501,59 @@ dump_info (const vcdinfo_obj_t *obj)
              info->first_seg_addr.m, info->first_seg_addr.s, info->first_seg_addr.f);
   
   if (gl.show.info.ofm)
-    fprintf (stdout, " offset multiplier: 0x%2.2x\n", info->offset_mult);
+    fprintf (stdout, " offset multiplier: 0x%2.2x\n", 
+             vcdinfo_get_offset_mult(obj));
 
   if (gl.show.info.lidn)
     fprintf (stdout, " maximum lid: %d\n",
              uint16_from_be (info->lot_entries));
 
   if (gl.show.info.segn)
-    fprintf (stdout, " maximum segment number: %d\n", num_segments);
+    fprintf (stdout, " number of segments: %d\n", num_segments);
   
   if (gl.show.info.segs)
     for (n = 0; n < num_segments; n++)
       {
-        fprintf (stdout, " SEGMENT[%d]: audio: %s,"
-                 " video: %s, continuation %s%s %s\n",
-                 n + 1,
+        const lsn_t  lsn = vcdinfo_get_seg_lsn(obj, n);
+        msf_t  msf;
+
+        cdio_lsn_to_msf(lsn, &msf);
+        fprintf (stdout, " SEGMENT[%4.4d]: track# 0, LSN %6d "
+                 "(MSF %2.2x:%2.2x:%2.2x), %2d sectors\n",
+                 n, lsn, msf.m, msf.s, msf.f, 
+                 vcdinfo_get_seg_sector_count(obj, n));
+
+        fprintf (stdout, "   audio: %s, video: %s, continuation %s%s %s\n",
                  vcdinfo_audio_type2str(obj,
                                         vcdinfo_get_seg_audio_type(obj, n)),
                  vcdinfo_video_type2str(obj, n),
                  _vcd_bool_str (info->spi_contents[n].item_cont),
-                 (obj->vcd_type == VCD_TYPE_VCD2) ? "" : ", OGT substream:",
-                 (obj->vcd_type == VCD_TYPE_VCD2) ? "" : vcdinfo_ogt2str(obj, n));
+                 (vcdinfo_get_VCD_type(obj) == VCD_TYPE_VCD2) 
+                   ? "" : ", OGT substream:",
+                 (vcdinfo_get_VCD_type(obj) == VCD_TYPE_VCD2) 
+                   ? "" : vcdinfo_ogt2str(obj, n));
       }
   
   if (gl.show.info.start)
-    if (obj->vcd_type == VCD_TYPE_SVCD
-        || obj->vcd_type == VCD_TYPE_HQVCD)
+    if (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD
+        || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
       for (n = 0; n < 5; n++)
         fprintf (stdout, " volume start time[%d]: %d secs\n", 
                  n, uint16_from_be (info->playing_time[n]));
 }
 
 static void
-dump_entries (const vcdinfo_obj_t *obj)
+dump_entries (vcdinfo_obj_t *obj)
 {
-  const EntriesVcd *entries = &obj->entries;
+  const EntriesVcd *entries = vcdinfo_get_entriesVcd(obj);
   int num_entries, n;
 
   num_entries = vcdinfo_get_num_entries(obj);
 
   if (!gl.show.no.header) 
     fprintf (stdout, 
-             (obj->vcd_type == VCD_TYPE_SVCD 
-              || obj->vcd_type == VCD_TYPE_HQVCD)
+             (vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+              || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
              ? "SVCD/ENTRIES.SVD\n"
              : "VCD/ENTRIES.VCD\n");
 
@@ -552,15 +577,15 @@ dump_entries (const vcdinfo_obj_t *obj)
   if (gl.show.entries.data) 
     for (n = 0; n < num_entries; n++)
       {
-        const msf_t *msf = vcdinfo_get_entry_msf(obj, n);
-        const lsn_t lsn = cdio_msf_to_lsn(msf);
-        
+        const lsn_t  lsn = vcdinfo_get_entry_lsn(obj, n);
+        msf_t  msf;
+
+        cdio_lsn_to_msf(lsn, &msf);
         fprintf (stdout, " ENTRY[%2.2d]: track# %2d (SEQUENCE[%d]), LSN %6d "
                  "(MSF %2.2x:%2.2x:%2.2x)\n",
                  n, vcdinfo_get_track(obj, n),
                  vcdinfo_get_track(obj, n) - 1,               
-                 lsn,
-                 msf->m, msf->s, msf->f);
+                 lsn, msf.m, msf.s, msf.f);
       }
 }
 
@@ -570,11 +595,11 @@ dump_entries (const vcdinfo_obj_t *obj)
    need TRACKS.SVCD
 */
 static void
-dump_tracks_svd (const vcdinfo_obj_t *obj)
+dump_tracks_svd (vcdinfo_obj_t *obj)
 {
-  const TracksSVD *tracks = obj->tracks_buf;
+  const TracksSVD *tracks = vcdinfo_get_tracksSVD(obj);
   const TracksSVD2 *tracks2 = (const void *) &(tracks->playing_time[tracks->tracks]);
-  const TracksSVD_v30 *tracks_v30 = obj->tracks_buf;
+  const TracksSVD_v30 *tracks_v30 = (TracksSVD_v30 *) tracks;
 
   unsigned j;
 
@@ -680,9 +705,9 @@ dump_tracks (const CdIo *cdio)
 }
 
 static void
-dump_scandata_dat (const vcdinfo_obj_t *obj)
+dump_scandata_dat (vcdinfo_obj_t *obj)
 {
-  const ScandataDat1 *_sd1 = obj->scandata_buf;
+  const ScandataDat1 *_sd1 = vcdinfo_get_scandata(obj);
   const uint16_t scandata_count = uint16_from_be (_sd1->scandata_count);
   const uint16_t track_count = uint16_from_be (_sd1->track_count);
   const uint16_t spi_count = uint16_from_be (_sd1->spi_count);
@@ -696,7 +721,7 @@ dump_scandata_dat (const vcdinfo_obj_t *obj)
 
   if (_sd1->version == SCANDATA_VERSION_VCD2)
     {
-      const ScandataDat_v2 *_sd_v2 = obj->scandata_buf;
+      const ScandataDat_v2 *_sd_v2 = (ScandataDat_v2 *) _sd1;
 
       int n;
 
@@ -710,8 +735,8 @@ dump_scandata_dat (const vcdinfo_obj_t *obj)
               && n < scandata_count - PRINTED_POINTS)
             continue;
 
-          fprintf (stdout, "  scanpoint[%.4d]: LSN %u (msf %2.2x:%2.2x:%2.2x)\n",
-                   n, lsn, msf->m, msf->s, msf->f);
+          fprintf (stdout, "  scanpoint[%.4d]: LSN %lu (msf %2.2x:%2.2x:%2.2x)\n",
+                   n, (long unsigned int) lsn, msf->m, msf->s, msf->f);
 
           if (!gl.verbose_flag
               && n == PRINTED_POINTS
@@ -780,8 +805,9 @@ dump_scandata_dat (const vcdinfo_obj_t *obj)
             continue;
 
           fprintf (stdout, 
-                   "  scanpoint[%.4d] (ofs:%5d): LSN %u (MSF %2.2x:%2.2x:%2.2x)\n",
-                   n, scandata_ofs0 + (n * 3), lsn, msf->m, msf->s, msf->f);
+                   "  scanpoint[%.4d] (ofs:%5d): LSN %lu (MSF %2.2x:%2.2x:%2.2x)\n",
+                   n, scandata_ofs0 + (n * 3), (unsigned long int) lsn, 
+                   msf->m, msf->s, msf->f);
 
           if (!gl.verbose_flag
               && n == PRINTED_POINTS
@@ -794,9 +820,9 @@ dump_scandata_dat (const vcdinfo_obj_t *obj)
 }
 
 static void
-dump_search_dat (const vcdinfo_obj_t *obj)
+dump_search_dat (vcdinfo_obj_t *obj)
 {
-  const SearchDat *searchdat = obj->search_buf;
+  const SearchDat *searchdat = vcdinfo_get_searchDat(obj);
   unsigned m;
   uint32_t scan_points = uint16_from_be (searchdat->scan_points);
 
@@ -804,8 +830,8 @@ dump_search_dat (const vcdinfo_obj_t *obj)
   fprintf (stdout, " ID: `%.8s'\n", searchdat->file_id);
   fprintf (stdout, " version: 0x%2.2x\n", searchdat->version);
   fprintf (stdout, " scanpoints: %d\n", scan_points);
-  fprintf (stdout, " scaninterval: %d (in 0.5sec units -- must be `1')\n", 
-           searchdat->time_interval);
+  fprintf (stdout, " scaninterval: %lu (in 0.5sec units -- must be `1')\n", 
+           (unsigned long int) searchdat->time_interval);
 
   for (m = 0; m < scan_points;m++)
     {
@@ -826,8 +852,8 @@ dump_search_dat (const vcdinfo_obj_t *obj)
       ss2 = (ss2 % 2) * 5;
 
       fprintf (stdout, " scanpoint[%.4d]: (real time: %.2d:%.2d:%.2d.%.1d) "
-               " sector: LSN %u (MSF %.2x:%.2x:%.2x)\n", m, hh, mm, ss, ss2,
-               lsn, msf->m, msf->s, msf->f);
+               " sector: LSN %lu (MSF %.2x:%.2x:%.2x)\n", m, hh, mm, ss, ss2,
+               (unsigned long int) lsn, msf->m, msf->s, msf->f);
       
       if (!gl.verbose_flag
           && m == PRINTED_POINTS && scan_points > (PRINTED_POINTS * 2))
@@ -839,9 +865,12 @@ dump_search_dat (const vcdinfo_obj_t *obj)
 static void
 _dump_fs_recurse (const vcdinfo_obj_t *obj, const char pathname[])
 {
-  VcdList *entlist = vcd_image_source_fs_readdir (obj->img, pathname);
+  VcdList *entlist;
   VcdList *dirlist =  _vcd_list_new ();
   VcdListNode *entnode;
+  CdIo *cdio = vcdinfo_get_cd_image(obj);
+
+  entlist = iso9660_fs_readdir (cdio, pathname, true);
     
   fprintf (stdout, " %s:\n", pathname);
 
@@ -853,11 +882,11 @@ _dump_fs_recurse (const vcdinfo_obj_t *obj, const char pathname[])
     {
       char *_name = _vcd_list_node_data (entnode);
       char _fullname[4096] = { 0, };
-      vcd_image_stat_t statbuf;
+      iso9660_stat_t statbuf;
 
       snprintf (_fullname, sizeof (_fullname), "%s%s", pathname, _name);
   
-      if (vcd_image_source_fs_stat (obj->img, _fullname, &statbuf))
+      if (iso9660_fs_stat (cdio, _fullname, &statbuf, true))
         vcd_assert_not_reached ();
 
       strncat (_fullname, "/", sizeof (_fullname));
@@ -868,21 +897,20 @@ _dump_fs_recurse (const vcdinfo_obj_t *obj, const char pathname[])
         _vcd_list_append (dirlist, strdup (_fullname));
 
       fprintf (stdout, 
-               "  %c %s %d %d [fn %.2d] [LSN %6d] ",
+               "  %c %s %d %d [fn %.2d] [LSN %6lu] ",
                (statbuf.type == _STAT_DIR) ? 'd' : '-',
-               vcdinfo_get_xa_attr_str (statbuf.xa.attributes),
+               iso9660_get_xa_attr_str (statbuf.xa.attributes),
                uint16_from_be (statbuf.xa.user_id),
                uint16_from_be (statbuf.xa.group_id),
                statbuf.xa.filenum,
-               statbuf.lsn);
+               (unsigned long int) statbuf.lsn);
 
       if (uint16_from_be(statbuf.xa.attributes) & XA_ATTR_MODE2FORM2) {
-        fprintf (stdout, "%9d (%9d)",
-                 statbuf.secsize * M2F2_SECTOR_SIZE,
-                 statbuf.size);
+        fprintf (stdout, "%9lu (%9lu)",
+                 (unsigned long int) statbuf.secsize * M2F2_SECTOR_SIZE,
+                 (unsigned long int) statbuf.size);
       } else {
-        fprintf (stdout, "%9d",
-                 statbuf.size);
+        fprintf (stdout, "%9lu", (unsigned long int) statbuf.size);
       }
       fprintf (stdout, "  %s\n", _name);
 
@@ -905,39 +933,39 @@ _dump_fs_recurse (const vcdinfo_obj_t *obj, const char pathname[])
 }
 
 static void
-dump_fs (const vcdinfo_obj_t *obj)
+dump_fs (vcdinfo_obj_t *obj)
 {
-  struct iso_primary_descriptor const *pvd = &obj->pvd;
-  struct iso_directory_record *idr = (void *) pvd->root_directory_record;
-  uint32_t extent;
-
-  extent = from_733 (idr->extent);
+  const iso9660_pvd_t *pvd = vcdinfo_get_pvd(obj);
+  const lsn_t extent = iso9660_get_root_lsn(pvd);
 
   fprintf (stdout, "ISO9660 filesystem dump\n");
-  fprintf (stdout, " root dir in PVD set to lsn %d\n\n", extent);
+  fprintf (stdout, " root directory in PVD set to LSN %lu\n\n", 
+           (unsigned long int) extent);
 
   _dump_fs_recurse (obj, "/");
 }
 
 static void
-dump_pvd (const vcdinfo_obj_t *obj)
+dump_pvd (vcdinfo_obj_t *obj)
 {
-  struct iso_primary_descriptor const *pvd = &obj->pvd;
+  const iso9660_pvd_t *pvd = vcdinfo_get_pvd(obj);
 
   if (!gl.show.no.header)
     fprintf (stdout, "ISO9660 primary volume descriptor\n");
 
-  if (pvd->type != ISO_VD_PRIMARY)
+  
+  if (iso9660_get_pvd_type(pvd) != ISO_VD_PRIMARY)
     vcd_warn ("unexpected descriptor type");
 
-  if (strncmp (pvd->id, ISO_STANDARD_ID, strlen (ISO_STANDARD_ID)))
+  if (strncmp (iso9660_get_pvd_id(pvd), ISO_STANDARD_ID, 
+               strlen (ISO_STANDARD_ID)))
     vcd_warn ("unexpected ID encountered (expected `" ISO_STANDARD_ID "'");
   
   if (gl.show.pvd.id)
-    fprintf (stdout, " ID: `%.5s'\n", pvd->id);
+    fprintf (stdout, " ID: `%.5s'\n", iso9660_get_pvd_id(pvd));
 
   if (gl.show.pvd.vers)
-    fprintf (stdout, " version: %d\n", pvd->version);
+    fprintf (stdout, " version: %d\n", iso9660_get_pvd_version(pvd));
 
   if (gl.show.pvd.sys)
     fprintf (stdout, " system id: `%s'\n",    vcdinfo_get_system_id(obj));
@@ -960,8 +988,8 @@ dump_pvd (const vcdinfo_obj_t *obj)
   
   if (gl.show.pvd.iso)
     fprintf (stdout, " ISO size: %d blocks (logical blocksize: %d bytes)\n", 
-             from_733 (pvd->volume_space_size),
-             from_723 (pvd->logical_block_size));
+             iso9660_get_pvd_space_size(pvd), 
+             iso9660_get_pvd_block_size(pvd));
 
   if (gl.show.pvd.xa) 
     fprintf (stdout, " XA marker present: %s\n", 
@@ -971,6 +999,8 @@ dump_pvd (const vcdinfo_obj_t *obj)
 static void
 dump_all (vcdinfo_obj_t *obj)
 {
+  CdIo *cdio = vcdinfo_get_cd_image(obj);
+
   if (gl.show.pvd.any) 
     {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
@@ -1010,7 +1040,7 @@ dump_all (vcdinfo_obj_t *obj)
           dump_psd (obj, false);
         }
       
-      if (obj->psd_x_size)
+      if (vcdinfo_get_psd_x_size(obj))
         {
           vcdinfo_visit_lot (obj, true);
           if (gl.show.lot) 
@@ -1025,18 +1055,18 @@ dump_all (vcdinfo_obj_t *obj)
 
   if (gl.show.tracks) 
     {
-      if (obj->tracks_buf)
+      if (vcdinfo_get_tracksSVD(obj))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
           dump_tracks_svd (obj);
         } 
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_tracks (obj->img);
+      dump_tracks (cdio);
     }
   
   if (gl.show.search) 
     {
-      if (obj->search_buf)
+      if (vcdinfo_get_searchDat(obj))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
           dump_search_dat (obj);
@@ -1046,7 +1076,7 @@ dump_all (vcdinfo_obj_t *obj)
 
   if (gl.show.scandata) 
     {
-      if (obj->scandata_buf)
+      if (vcdinfo_get_scandata(obj))
         {
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
           dump_scandata_dat (obj);
@@ -1057,13 +1087,13 @@ dump_all (vcdinfo_obj_t *obj)
 
 }
 
-
 static void
 dump (char image_fname[])
 {
   unsigned size, psd_size;
-  vcdinfo_obj_t obj;
-  vcd_image_stat_t statbuf;
+  vcdinfo_obj_t *obj;
+  CdIo *img;
+  iso9660_stat_t statbuf;
   vcdinfo_open_return_t open_rc;
   
   if (!gl.show.no.banner)
@@ -1075,23 +1105,24 @@ dump (char image_fname[])
                "%s\n\n", _rcsid);
     }
 
-  if (VCDINFO_SOURCE_UNDEF==gl.source_type) 
-    {
-      gl.source_type = VCDINFO_SOURCE_DEVICE;
-    }
-  
+  if (NULL == obj) {
+    fprintf (stdout, "error allocating a new VCD object\n");
+    exit (EXIT_FAILURE);
+  }
+
   open_rc = vcdinfo_open(&obj, &image_fname, gl.source_type, gl.access_mode);
-  if (open_rc==VCDINFO_OPEN_ERROR || obj.img == NULL) {
+  img = vcdinfo_get_cd_image(obj);
+  if (open_rc==VCDINFO_OPEN_ERROR || img == NULL) {
     vcd_error ("Error determining place to read from");
     exit (EXIT_FAILURE);
   }
 
-  size = cdio_stat_size (obj.img);
+  size = cdio_stat_size (img);
 
   if (gl.show.source) 
     {
       if (NULL == image_fname) {
-        image_fname = vcdinfo_get_default_device(&obj);
+        image_fname = vcdinfo_get_default_device(obj);
         fprintf (stdout, "Source: default image file `%s'\n", image_fname);
         free(image_fname);
       } else {
@@ -1104,42 +1135,42 @@ dump (char image_fname[])
     vcd_warn ("Medium is not VCD image");
     if (gl.show.fs) 
       {
-        if (vcdinfo_has_xa(&obj))
+        if (vcdinfo_has_xa(obj))
         {
           /* Suppress XA warnings */
           int old_suppress_warnings = gl.suppress_warnings;
           if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
           gl.suppress_warnings=1;
-          dump_fs (&obj);
+          dump_fs (obj);
           gl.suppress_warnings=old_suppress_warnings;
         }
       }
 
     if (gl.show.tracks) {
       if (!gl.show.no.delimiter) fprintf (stdout, DELIM);
-      dump_tracks (obj.img);
+      dump_tracks (img);
     }
     exit (EXIT_FAILURE);
   }
 
-  if (vcdinfo_get_format_version (&obj) == VCD_TYPE_INVALID) {
+  if (vcdinfo_get_format_version (obj) == VCD_TYPE_INVALID) {
     vcd_error ("VCD detection failed - aborting");
     exit (EXIT_FAILURE);
   } else if (gl.show.format) {
-    fprintf (stdout, "%s detected\n", vcdinfo_get_format_version_str(&obj));
+    fprintf (stdout, "%s detected\n", vcdinfo_get_format_version_str(obj));
   }
 
-  psd_size = vcdinfo_get_psd_size (&obj);
+  psd_size = vcdinfo_get_psd_size (obj);
 
-  if (vcdinfo_read_psd (&obj))
+  if (vcdinfo_read_psd (obj))
     {
       /* ISO9660 crosscheck */
-      if (vcd_image_source_fs_stat (obj.img, 
-                                    ((obj.vcd_type == VCD_TYPE_SVCD 
-                                      || obj.vcd_type == VCD_TYPE_HQVCD)
-                                     ? "/SVCD/PSD.SVD;1" 
-                                     : "/VCD/PSD.VCD;1"),
-                                    &statbuf))
+      if (iso9660_fs_stat (img, 
+                           ((vcdinfo_get_VCD_type(obj) == VCD_TYPE_SVCD 
+                             || vcdinfo_get_VCD_type(obj) == VCD_TYPE_HQVCD)
+                            ? "/SVCD/PSD.SVD;1" 
+                            : "/VCD/PSD.VCD;1"),
+                           &statbuf, true))
         vcd_warn ("no PSD file entry found in ISO9660 fs");
       else
         {
@@ -1151,90 +1182,8 @@ dump (char image_fname[])
           
     }
 
-  if (obj.vcd_type == VCD_TYPE_SVCD
-      || obj.vcd_type == VCD_TYPE_HQVCD)
-    {
-      if (!vcd_image_source_fs_stat (obj.img, "MPEGAV", &statbuf))
-        vcd_warn ("non compliant /MPEGAV folder detected!");
-
-      if (vcd_image_source_fs_stat (obj.img, "SVCD/TRACKS.SVD;1", &statbuf))
-        vcd_warn ("mandatory /SVCD/TRACKS.SVD not found!");
-      else
-        vcd_debug ("found TRACKS.SVD signature at sector %d", statbuf.lsn);
-
-      if (vcd_image_source_fs_stat (obj.img, "SVCD/SEARCH.DAT;1", &statbuf))
-        vcd_warn ("mandatory /SVCD/SEARCH.DAT not found!");
-      else
-        {
-          uint32_t size;
-          vcd_debug ("found SEARCH.DAT at sector %d", statbuf.lsn);
-
-          obj.search_buf = _vcd_malloc (ISO_BLOCKSIZE * statbuf.secsize);
-
-          if (cdio_read_mode2_sectors (obj.img, obj.search_buf, statbuf.lsn, 
-                                       false, statbuf.secsize))
-            exit (EXIT_FAILURE);
-
-          size = (3 * uint16_from_be (((SearchDat *)obj.search_buf)->scan_points)) 
-            + sizeof (SearchDat);
-          
-          if (size > statbuf.size)
-            {
-              vcd_warn ("number of scanpoints leads to bigger size than "
-                        "file size of SEARCH.DAT! -- rereading");
-
-              free (obj.search_buf);
-              obj.search_buf = _vcd_malloc (ISO_BLOCKSIZE * _vcd_len2blocks (size, ISO_BLOCKSIZE));
-          
-              if (cdio_read_mode2_sectors (obj.img, obj.search_buf,statbuf.lsn,
-                                           false, statbuf.secsize))
-                exit (EXIT_FAILURE);
-            }
-        }
-    }
-
-  if (!vcd_image_source_fs_stat (obj.img, "EXT/SCANDATA.DAT;1", &statbuf))
-    {
-      vcd_debug ("found /EXT/SCANDATA.DAT at sector %d", statbuf.lsn);
-
-      obj.scandata_buf = _vcd_malloc (ISO_BLOCKSIZE * statbuf.secsize);
-
-      if (cdio_read_mode2_sectors (obj.img, obj.scandata_buf, statbuf.lsn, 
-                                   false, statbuf.secsize))
-        exit (EXIT_FAILURE);
-    }
-
-  if (obj.vcd_type == VCD_TYPE_VCD2)
-    {
-      if (!vcd_image_source_fs_stat (obj.img, "EXT/PSD_X.VCD;1", &statbuf))
-        {
-          vcd_debug ("found /EXT/PSD_X.VCD at sector %d", statbuf.lsn);
-
-          obj.psd_x = _vcd_malloc (ISO_BLOCKSIZE * statbuf.secsize);
-          obj.psd_x_size = statbuf.size;
-
-          if (cdio_read_mode2_sectors (obj.img, obj.psd_x, statbuf.lsn, false, 
-                                       statbuf.secsize))
-            exit (EXIT_FAILURE);
-        }
-
-      if (!vcd_image_source_fs_stat (obj.img, "EXT/LOT_X.VCD;1", &statbuf))
-        {
-          vcd_debug ("found /EXT/LOT_X.VCD at sector %d", statbuf.lsn);
-
-          obj.lot_x = _vcd_malloc (ISO_BLOCKSIZE * statbuf.secsize);
-
-          if (cdio_read_mode2_sectors (obj.img, obj.lot_x, statbuf.lsn, false, 
-                                                   statbuf.secsize))
-            exit (EXIT_FAILURE);
-
-          if (statbuf.size != LOT_VCD_SIZE * ISO_BLOCKSIZE)
-            vcd_warn ("LOT_X.VCD size != 65535");
-        }
-    }
-
-  dump_all (&obj);
-  vcdinfo_close(&obj);
+  dump_all (obj);
+  vcdinfo_close(obj);
 }
 
 static vcd_log_handler_t  gl_default_vcd_log_handler = NULL;
@@ -1258,14 +1207,12 @@ _vcd_log_handler (vcd_log_level_t level, const char message[])
 /* Configuration option codes */
 enum {
 
-  /* These correspond to vcdinfo_source_t in vcdinfo.h and have to MATCH! */
-  OP_SOURCE_UNDEF       = VCDINFO_SOURCE_UNDEF,
-  OP_SOURCE_AUTO        = VCDINFO_SOURCE_AUTO,
-  OP_SOURCE_BIN         = VCDINFO_SOURCE_BIN,
-  OP_SOURCE_CUE         = VCDINFO_SOURCE_CUE,
-  OP_SOURCE_NRG         = VCDINFO_SOURCE_NRG,
-  OP_SOURCE_DEVICE      = VCDINFO_SOURCE_DEVICE,
-  OP_SOURCE_SECTOR_2336 = VCDINFO_SOURCE_SECTOR_2336,
+  /* These correspond to driver_id_t in cdio.h and have to MATCH! */
+  OP_SOURCE_UNDEF       = DRIVER_UNKNOWN,
+  OP_SOURCE_BINCUE      = DRIVER_BINCUE,
+  OP_SOURCE_NRG         = DRIVER_NRG,
+  OP_SOURCE_DEVICE      = DRIVER_DEVICE,
+  OP_SOURCE_SECTOR_2336,
 
   /* These are the remaining configuration options */
   OP_VERSION,       OP_ENTRIES,   OP_INFO,      OP_PVD,       OP_SHOW, 
@@ -1279,7 +1226,7 @@ init()
 {
   gl.verbose_flag     = false;
   gl.quiet_flag       = false;
-  gl.source_type      = VCDINFO_SOURCE_UNDEF;
+  gl.source_type      = DRIVER_UNKNOWN;
   gl.access_mode      = NULL;
 
   /* Set all of show-flag entries false in one go. */
@@ -1364,13 +1311,13 @@ main (int argc, const char *argv[])
      "set CD-ROM access mode (IOCTL, READ_10, READ_CD)", "ACCESS"},
 
     {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
-     OP_SOURCE_BIN, "set \"bin\" CD-ROM disk image file as source", "FILE"},
+     OP_SOURCE_BINCUE, "set \"bin\" CD-ROM disk image file as source", "FILE"},
 
     {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
-     OP_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
+     OP_SOURCE_BINCUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
 
     {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
-     OP_SOURCE_AUTO,
+     OP_SOURCE_UNDEF,
      "set source and determine if \"bin\" image or device", "FILE"},
 
     {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
@@ -1541,9 +1488,8 @@ main (int argc, const char *argv[])
         /* Make sure a we do only once? */
         break;
 
-      case OP_SOURCE_AUTO:
-      case OP_SOURCE_BIN: 
-      case OP_SOURCE_CUE: 
+      case OP_SOURCE_UNDEF:
+      case OP_SOURCE_BINCUE: 
       case OP_SOURCE_NRG: 
       case OP_SOURCE_DEVICE: 
       case OP_SOURCE_SECTOR_2336:
@@ -1556,7 +1502,7 @@ main (int argc, const char *argv[])
             okay = true;
             gl.source_type = opt;
             break;
-          case OP_SOURCE_BIN:
+          case OP_SOURCE_BINCUE:
             /* Going from 2352 (default) to 2336 is okay. */
             okay = OP_SOURCE_SECTOR_2336 == opt;
             if (okay) 
@@ -1568,8 +1514,6 @@ main (int argc, const char *argv[])
              */
             okay = OP_SOURCE_DEVICE != opt;
             break;
-          case OP_SOURCE_AUTO:
-          case OP_SOURCE_CUE:
           case OP_SOURCE_NRG: 
           case OP_SOURCE_DEVICE:
             /* This case is implied, but we'll make it explicit anyway. */
