@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_xml_rip.c,v 1.52.2.13 2003/05/18 01:53:09 rocky Exp $
+    $Id: vcd_xml_rip.c,v 1.52.2.31 2003/10/03 21:28:51 rocky Exp $
 
     Copyright (C) 2001,2003 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -18,9 +18,27 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+/* Private headers */
+#include "vcd_read.h"
+#include "bytesex.h"
+#include "vcdxml.h"
+#include "vcd_xml_dtd.h"
+#include "vcd_xml_dump.h"
+#include "vcd_xml_common.h"
+
+
+/* FIXME: Make this really private: */
+#include <libvcd/files_private.h>
+
+/* Public headers */
+
+#include <cdio/iso9660.h>
+#include <cdio/cd_types.h>
+#include <cdio/logging.h>
+
+#include <libvcd/sector.h>
+#include <libvcd/files.h>
+#include <libvcd/info.h>
 
 #include <stdio.h>
 #include <errno.h>
@@ -40,28 +58,14 @@
 
 #include <popt.h>
 
-#include <libvcd/sector.h>
-#include <libvcd/files.h>
-#include <libvcd/info.h>
-#include <libvcd/info_private.h>
-#include <libvcd/iso9660_private.h>
-
-/* Private headers */
-#include "image_fs.h"
-
-#include "vcdxml.h"
-#include "vcd_xml_dtd.h"
-#include "vcd_xml_dump.h"
-#include "vcd_xml_common.h"
-
-static const char _rcsid[] = "$Id: vcd_xml_rip.c,v 1.52.2.13 2003/05/18 01:53:09 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd_xml_rip.c,v 1.52.2.31 2003/10/03 21:28:51 rocky Exp $";
 
 static int _verbose_flag = 0;
 static int _quiet_flag = 0;
 
 static void
 _register_file (struct vcdxml_t *obj, const char *pathname,
-		vcd_image_stat_t const *statbuf)
+		iso9660_stat_t const *statbuf)
 {
   uint16_t xa_attr = uint16_from_be (statbuf->xa.attributes);
 
@@ -135,7 +139,7 @@ _register_file (struct vcdxml_t *obj, const char *pathname,
 static int
 _parse_isofs_r (struct vcdxml_t *obj, CdIo *img, const char pathname[])
 { 
-  VcdList *entlist = vcd_image_source_fs_readdir (img, pathname);
+  VcdList *entlist = iso9660_fs_readdir (img, pathname, true);
   VcdListNode *entnode;
 
   if (entlist == NULL)
@@ -145,12 +149,12 @@ _parse_isofs_r (struct vcdxml_t *obj, CdIo *img, const char pathname[])
     {
       char *_name = _vcd_list_node_data (entnode);
       char _fullname[4096] = { 0, };
-      vcd_image_stat_t statbuf;
+      iso9660_stat_t statbuf;
 
       strncpy (_fullname, pathname, sizeof (_fullname));
       strncat (_fullname, _name, sizeof (_fullname));
 
-      if (vcd_image_source_fs_stat (img, _fullname, &statbuf))
+      if (iso9660_fs_stat (img, _fullname, &statbuf, true))
 	return -1;
       
       if (!strcmp (_name, ".") 
@@ -188,31 +192,15 @@ _parse_isofs (struct vcdxml_t *obj, CdIo *img)
 static int
 _parse_pvd (struct vcdxml_t *obj, CdIo *img)
 {
-  struct iso_primary_descriptor pvd;
+  iso9660_pvd_t pvd;
 
-  memset (&pvd, 0, sizeof (struct iso_primary_descriptor));
-  vcd_assert (sizeof (struct iso_primary_descriptor) == ISO_BLOCKSIZE);
+  memset (&pvd, 0, sizeof (iso9660_pvd_t));
+  vcd_assert (sizeof (iso9660_pvd_t) == ISO_BLOCKSIZE);
 
-  if (cdio_read_mode2_sector (img, &pvd, ISO_PVD_SECTOR, false) != 0) 
-    {
-      vcd_error ("error reading PVD sector (%d)", ISO_PVD_SECTOR);
-      return -1;
-    }
-  
+  if (!read_pvd(img, &pvd)) {
+    return -1;
+  }
 
-  if (pvd.type != ISO_VD_PRIMARY)
-    {
-      vcd_error ("unexpected descriptor type");
-      return -1;
-    }
-  
-  if (strncmp (pvd.id, ISO_STANDARD_ID, strlen (ISO_STANDARD_ID)))
-    {
-      vcd_error ("unexpected ID encountered (expected `"
-		 ISO_STANDARD_ID "', got `%.5s'", pvd.id);
-      return -1;
-    }
- 
   obj->pvd.volume_id      = strdup (vcdinf_get_volume_id(&pvd));
   obj->pvd.system_id      = strdup (vcdinf_get_system_id(&pvd));
   obj->pvd.publisher_id   = strdup (vcdinf_get_publisher_id(&pvd));
@@ -226,34 +214,15 @@ static int
 _parse_info (struct vcdxml_t *obj, CdIo *img)
 {
   InfoVcd info;
-  vcd_type_t _vcd_type;
 
   memset (&info, 0, sizeof (InfoVcd));
   vcd_assert (sizeof (InfoVcd) == ISO_BLOCKSIZE);
 
-  cdio_read_mode2_sector (img, &info, INFO_VCD_SECTOR, false);
+  if (!read_info(img, &info, &(obj->vcd_type)))
+    return -1;
 
-  /* analyze signature/type */
-
-  obj->vcd_type = _vcd_type = vcd_files_info_detect_type (&info);
-
-  switch (_vcd_type)
-    {
-    case VCD_TYPE_VCD:
-    case VCD_TYPE_VCD11:
-    case VCD_TYPE_VCD2:
-    case VCD_TYPE_SVCD:
-    case VCD_TYPE_HQVCD:
-      vcd_debug ("%s detected", vcdinf_get_format_version_str(_vcd_type));
-      break;
-    case VCD_TYPE_INVALID:
-      vcd_error ("unknown ID encountered -- maybe not a proper (S)VCD?");
-      return -1;
-      break;
-    default:
-      vcd_assert_not_reached ();
-      break;
-    }
+  if (obj->vcd_type == VCD_TYPE_INVALID)
+    return -1;
 
   obj->info.album_id = strdup (vcdinf_get_album_id(&info));
   obj->info.volume_count = vcdinf_get_volume_count(&info);
@@ -268,7 +237,7 @@ _parse_info (struct vcdxml_t *obj, CdIo *img)
 
   {
     lba_t segment_start;
-    unsigned max_seg_num;
+    segnum_t max_seg_num;
     int idx, n;
     struct segment_t *_segment = NULL;
 
@@ -322,24 +291,14 @@ _parse_entries (struct vcdxml_t *obj, CdIo *img)
 {
   EntriesVcd entries;
   int idx;
-  uint8_t ltrack;
+  track_t ltrack;
 
   memset (&entries, 0, sizeof (EntriesVcd));
   vcd_assert (sizeof (EntriesVcd) == ISO_BLOCKSIZE);
 
-  cdio_read_mode2_sector (img, &entries, ENTRIES_VCD_SECTOR, false);
-
-  /* analyze signature/type */
-
-  if (!strncmp (entries.ID, ENTRIES_ID_VCD, sizeof (entries.ID)))
-    {}
-  else if (!strncmp (entries.ID, "ENTRYSVD", sizeof (entries.ID)))
-    vcd_warn ("found (non-compliant) SVCD ENTRIES.SVD signature");
-  else
-    {
-      vcd_error ("unexpected ID signature encountered `%.8s'", entries.ID);
-      return -1;
-    }
+  if (!read_entries(img, &entries)) {
+    return -1;
+  }
 
   ltrack = 0;
   for (idx = 0; idx < vcdinf_get_num_entries(&entries); idx++)
@@ -437,7 +396,7 @@ _pin2id (unsigned pin)
     snprintf (buf, sizeof (buf), "entry-%3.3d", itemid.num);
     break;
   case VCDINFO_ITEM_TYPE_SEGMENT:
-    snprintf (buf, sizeof (buf), "segment-%4.4d", itemid.num-1);
+    snprintf (buf, sizeof (buf), "segment-%4.4d", itemid.num);
     break;
   case VCDINFO_ITEM_TYPE_SPAREID2:
   default:
@@ -540,20 +499,20 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
       _pbc = vcd_pbc_new (PBC_PLAYLIST);
       {
 	const PsdPlayListDescriptor *d = (const void *) _buf;
-	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_get_prev_from_pld(d), 
+	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_pld_get_prev_offset(d), 
 					   _ctx));
-	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_get_next_from_pld(d),
+	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_pld_get_next_offset(d),
 					   _ctx));
-	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_get_return_from_pld(d), 
+	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_pld_get_return_offset(d),
 					   _ctx));
 	
 	_pbc->playing_time = (double) (vcdinf_get_play_time(d)) / 15.0;
-	_pbc->wait_time       = vcdinfo_get_wait_time(d);
-	_pbc->auto_pause_time = vcdinfo_get_autowait_time(d);
+	_pbc->wait_time       = vcdinf_get_wait_time(d);
+	_pbc->auto_pause_time = vcdinf_get_autowait_time(d);
 
-	for (n = 0; n < d->noi; n++) {
+	for (n = 0; n < vcdinf_pld_get_noi(d); n++) {
 	  _vcd_list_append (_pbc->item_id_list, 
-			    _xstrdup(_pin2id(vcdinf_get_play_item_from_pld(d,n)
+			    _xstrdup(_pin2id(vcdinf_pld_get_play_item(d,n)
 					     )));
 	  }
       }
@@ -565,14 +524,14 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
       {
 	const PsdSelectionListDescriptor *d = (const void *) _buf;
 	_pbc->bsn = vcdinf_get_bsn(d);
-	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_get_prev_from_psd(d), 
+	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_psd_get_prev_offset(d), 
 					   _ctx));
-	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_get_next_from_psd(d), 
+	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_psd_get_next_offset(d), 
 					   _ctx));
-	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_get_return_from_psd(d), 
+	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_psd_get_return_offset(d),
 					   _ctx));
 
-	switch (vcdinf_get_default_from_psd(d))
+	switch (vcdinf_psd_get_default_offset(d))
 	  {
 	  case PSD_OFS_DISABLED:
 	    _pbc->default_id = NULL;
@@ -590,22 +549,23 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
 	    break;
 
 	  default:
-	    _pbc->default_id = _xstrdup (_ofs2id (uint16_from_be (d->default_ofs), _ctx));
+	    _pbc->default_id = _xstrdup (_ofs2id (vcdinf_psd_get_default_offset(d),
+						  _ctx));
 	    _pbc->selection_type = _SEL_NORMAL;
 	    break;
 	  }
 
-	_pbc->timeout_id = 
-	  _xstrdup (_ofs2id (uint16_from_be (d->timeout_ofs), _ctx));
-	_pbc->timeout_time = vcdinfo_get_timeout_time (d);
+	_pbc->timeout_id   = _xstrdup (_ofs2id (vcdinf_get_timeout_offset(d), 
+						_ctx));
+	_pbc->timeout_time = vcdinf_get_timeout_time (d);
 	_pbc->jump_delayed = vcdinf_has_jump_delay(d);
-	_pbc->loop_count = vcdinf_get_loop_count(d);
-	_pbc->item_id = _xstrdup (_pin2id (uint16_from_be (d->itemid)));
+	_pbc->loop_count   = vcdinf_get_loop_count(d);
+	_pbc->item_id      = _xstrdup (_pin2id (vcdinf_psd_get_itemid(d)));
 
 	for (n = 0; n < vcdinf_get_num_selections(d); n++)
 	  {
 	    _vcd_list_append (_pbc->select_id_list, 
-			      _xstrdup(_ofs2id(vcdinf_get_offset_from_psd(d,n),
+			      _xstrdup(_ofs2id(vcdinf_psd_get_offset(d,n),
 					       _ctx)));
 	  }
 
@@ -732,7 +692,7 @@ _visit_pbc (struct _pbc_ctx *obj, lid_t lid, unsigned int offset, bool in_lot)
       _vcd_list_append (obj->offset_list, ofs);
       {
         const PsdPlayListDescriptor *d = (const void *) (obj->psd + _rofs);
-        const lid_t lid = vcdinf_get_lid_from_pld(d);
+        const lid_t lid = vcdinf_pld_get_lid(d);
 
         if (!ofs->lid)
           ofs->lid = lid;
@@ -741,9 +701,9 @@ _visit_pbc (struct _pbc_ctx *obj, lid_t lid, unsigned int offset, bool in_lot)
             vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
                       ofs->lid, lid);
 
-        _visit_pbc (obj, 0, vcdinf_get_prev_from_pld(d), false);
-        _visit_pbc (obj, 0, vcdinf_get_next_from_pld(d), false);
-        _visit_pbc (obj, 0, vcdinf_get_return_from_pld(d), false);
+        _visit_pbc (obj, 0, vcdinf_pld_get_prev_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_pld_get_next_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_pld_get_return_offset(d), false);
       }
       break;
 
@@ -755,22 +715,23 @@ _visit_pbc (struct _pbc_ctx *obj, lid_t lid, unsigned int offset, bool in_lot)
           (const void *) (obj->psd + _rofs);
 
         int idx;
+	lid_t lid = vcdinf_psd_get_lid (d);
 
         if (!ofs->lid)
-          ofs->lid = uint16_from_be (d->lid) & 0x7fff;
+          ofs->lid = lid;
         else 
-          if (ofs->lid != (uint16_from_be (d->lid) & 0x7fff))
+          if (ofs->lid != lid)
             vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
-                      ofs->lid, uint16_from_be (d->lid) & 0x7fff);
+                      ofs->lid, lid);
 
-        _visit_pbc (obj, 0, vcdinf_get_prev_from_psd(d), false);
-        _visit_pbc (obj, 0, vcdinf_get_next_from_psd(d), false);
-        _visit_pbc (obj, 0, vcdinf_get_return_from_psd(d), false);
-        _visit_pbc (obj, 0, vcdinf_get_default_from_psd(d), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->timeout_ofs), false);
+        _visit_pbc (obj, 0, vcdinf_psd_get_prev_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_psd_get_next_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_psd_get_return_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_psd_get_default_offset(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_timeout_offset(d), false);
 
         for (idx = 0; idx < vcdinf_get_num_selections(d); idx++)
-          _visit_pbc (obj, 0, vcdinf_get_offset_from_psd (d, idx), false);
+          _visit_pbc (obj, 0, vcdinf_psd_get_offset (d, idx), false);
         
       }
       break;
@@ -810,7 +771,7 @@ _parse_pbc (struct vcdxml_t *obj, CdIo *img, bool no_ext_psd)
   uint32_t _lot_vcd_sector = -1;
   uint32_t _psd_vcd_sector = -1;
   unsigned _psd_size = -1;
-  vcd_image_stat_t statbuf;
+  iso9660_stat_t statbuf;
 
   if (!obj->info.psd_size)
     {
@@ -820,7 +781,7 @@ _parse_pbc (struct vcdxml_t *obj, CdIo *img, bool no_ext_psd)
 
   if (obj->vcd_type == VCD_TYPE_VCD2)
     {
-      if (!vcd_image_source_fs_stat (img, "EXT/LOT_X.VCD;1", &statbuf))
+      if (!iso9660_fs_stat (img, "EXT/LOT_X.VCD;1", &statbuf, true))
 	{
 	  extended = true;
 	  _lot_vcd_sector = statbuf.lsn;
@@ -828,7 +789,7 @@ _parse_pbc (struct vcdxml_t *obj, CdIo *img, bool no_ext_psd)
 	}  
 
       if (extended &&
-	  !vcd_image_source_fs_stat (img, "EXT/PSD_X.VCD;1", &statbuf))
+	  !iso9660_fs_stat (img, "EXT/PSD_X.VCD;1", &statbuf, true))
 	{
 	  _psd_vcd_sector = statbuf.lsn;
 	  _psd_size = statbuf.size;
@@ -953,7 +914,7 @@ static int
 _rip_segments (struct vcdxml_t *obj, CdIo *img)
 {
   VcdListNode *node;
-  uint32_t start_extent;
+  lsn_t start_extent;
 
   start_extent = obj->info.segments_start;
 
@@ -980,7 +941,7 @@ _rip_segments (struct vcdxml_t *obj, CdIo *img)
           exit (EXIT_FAILURE);
         }
 
-      for (n = 0; n < _seg->segments_count * 150; n++)
+      for (n = 0; n < _seg->segments_count * VCDINFO_SEGMENT_SECTOR_SIZE; n++)
 	{
 	  struct m2f2sector
           {
@@ -1038,7 +999,7 @@ _rip_segments (struct vcdxml_t *obj, CdIo *img)
 
       fclose (outfd);
 
-      start_extent += _seg->segments_count * 150;
+      start_extent += _seg->segments_count * VCDINFO_SEGMENT_SECTOR_SIZE;
     }
 
   return 0;
@@ -1114,7 +1075,7 @@ _rip_sequences (struct vcdxml_t *obj, CdIo *img, int tracknum)
 							   ? 15 : secs_left));
 	    }
 
-	  if (_nseq && n + 150 == end_lsn + 1)
+	  if (_nseq && n + CDIO_POSTGAP_SECTORS == end_lsn + 1)
 	    vcd_warn ("reading into gap @%d... :-(", n);
 
 	  if (!(buf[buf_idx].subheader[2] & SM_FORM2))
@@ -1243,6 +1204,21 @@ _rip_sequences (struct vcdxml_t *obj, CdIo *img, int tracknum)
   return 0;
 }
 
+static vcd_log_handler_t  gl_default_vcd_log_handler = NULL;
+static cdio_log_handler_t gl_default_cdio_log_handler = NULL;
+
+static void 
+_vcd_log_handler (vcd_log_level_t level, const char message[])
+{
+  if (level == VCD_LOG_DEBUG && !_verbose_flag)
+    return;
+
+  if (level == VCD_LOG_INFO && _quiet_flag)
+    return;
+  
+  gl_default_vcd_log_handler (level, message);
+}
+
 #define DEFAULT_XML_FNAME      "videocd.xml"
 #define DEFAULT_IMG_FNAME      "videocd.bin"
 
@@ -1267,12 +1243,10 @@ main (int argc, const char *argv[])
   int _track_flag=0;
 
   enum { 
-    CL_SOURCE_UNDEF = VCDINFO_SOURCE_UNDEF,
-    CL_SOURCE_AUTO  = VCDINFO_SOURCE_AUTO,
-    CL_SOURCE_BIN   = VCDINFO_SOURCE_BIN,
-    CL_SOURCE_CUE   = VCDINFO_SOURCE_CUE,
-    CL_SOURCE_NRG   = VCDINFO_SOURCE_NRG,
-    CL_SOURCE_CDROM = VCDINFO_SOURCE_DEVICE,
+    CL_SOURCE_UNDEF = DRIVER_UNKNOWN, 
+    CL_SOURCE_BINCUE= DRIVER_BINCUE,
+    CL_SOURCE_NRG   = DRIVER_NRG,
+    CL_SOURCE_CDROM = DRIVER_DEVICE,
     CL_VERSION      = 20
   } _img_type = CL_SOURCE_UNDEF;
 
@@ -1280,7 +1254,8 @@ main (int argc, const char *argv[])
 
   vcd_xml_init (&obj);
 
-  vcd_xml_log_init ();
+  gl_default_vcd_log_handler  = vcd_log_set_handler (_vcd_log_handler);
+  gl_default_cdio_log_handler = cdio_log_set_handler (_vcd_log_handler);
 
   {
     poptContext optCon = NULL;
@@ -1292,12 +1267,12 @@ main (int argc, const char *argv[])
        "FILE"},
 
       {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
-       CL_SOURCE_BIN,
+       CL_SOURCE_BINCUE,
        "set image file as source (default: '" DEFAULT_IMG_FNAME "')", 
        "FILE"},
 
       {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
-       CL_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
+       CL_SOURCE_BINCUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
 
       {"sector-2336", '\0', POPT_ARG_NONE, &sector_2336_flag, 0,
        "use 2336 byte sector mode for image file"},
@@ -1306,7 +1281,7 @@ main (int argc, const char *argv[])
        CL_SOURCE_CDROM,"set CDROM device as source", "DEVICE"},
 
       {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
-       CL_SOURCE_AUTO, 
+       CL_SOURCE_UNDEF, 
        "set source and determine if \"bin\" image or device", "FILE"},
 
       {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
@@ -1334,6 +1309,12 @@ main (int argc, const char *argv[])
       {"progress", 'p', POPT_ARG_NONE, &_progress_flag, 0,  
        "show progress"}, 
 
+      { "track", 't', POPT_ARG_INT, &_track_flag, 0,
+	"rip only this track"},
+      
+      { "filename-encoding", '\0', POPT_ARG_STRING, &vcd_xml_filename_charset, 0,
+        "use given charset encoding for filenames instead of UTF8" },
+
       {"verbose", 'v', POPT_ARG_NONE, &_verbose_flag, 0, 
        "be verbose"},
     
@@ -1345,14 +1326,11 @@ main (int argc, const char *argv[])
       {"version", 'V', POPT_ARG_NONE, NULL, CL_VERSION,
        "display version and copyright information and exit"},
 	   
-      { "track", 't', POPT_ARG_INT, &_track_flag, 0,
-	"rip only this track"},
-      
       POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
     };
 
     optCon = poptGetContext ("vcdimager", argc, argv, optionsTable, 0);
-      
+
     if (poptReadDefaultConfig (optCon, 0)) 
       vcd_warn ("reading popt configuration failed"); 
 
@@ -1365,10 +1343,8 @@ main (int argc, const char *argv[])
 	  exit (EXIT_SUCCESS);
 	  break;
 
-	case CL_SOURCE_AUTO:
 	case CL_SOURCE_NRG:
-	case CL_SOURCE_BIN:
-	case CL_SOURCE_CUE: 
+	case CL_SOURCE_BINCUE:
 	case CL_SOURCE_CDROM:
 	  if (_img_type)
 	    {
@@ -1412,15 +1388,30 @@ main (int argc, const char *argv[])
   if (!xml_fname)
     xml_fname = strdup (DEFAULT_XML_FNAME);
 
-  if (CL_SOURCE_UNDEF == _img_type) 
-    _img_type = CL_SOURCE_AUTO;
+  /* If we don't specify a driver_id or a source_name, scan the
+     system for a CD that contains a VCD.
+   */
+  if (NULL == img_fname && _img_type == DRIVER_UNKNOWN) {
+    char **cd_drives=NULL;
+    cd_drives = cdio_get_devices_with_cap(NULL, 
+                (CDIO_FS_ANAL_SVCD|CDIO_FS_ANAL_CVD|CDIO_FS_ANAL_VIDEOCD
+                |CDIO_FS_UNKNOWN),
+                                          true);
+    if ( NULL == cd_drives || NULL == cd_drives[0] ) {
+      return VCDINFO_OPEN_ERROR;
+    }
+    img_fname = strdup(cd_drives[0]);
+    cdio_free_device_list(cd_drives);
+  }
 
-  if (!vcdinf_open(&img_src, &img_fname, _img_type, NULL)) {
+  img_src = cdio_open(img_fname, _img_type);
+  if (NULL == img_src) {
     vcd_error ("Error determining place to read from.");
     exit (EXIT_FAILURE);
   }
   
-  vcd_assert (img_src != NULL);
+  if (NULL == img_fname) 
+    img_fname = cdio_get_default_device(img_src);
 
   obj.comment = vcd_xml_dump_cl_comment (argc, argv, nocommand_comment_flag);
 
