@@ -1,5 +1,5 @@
 /*
-  $Id: mpeg.c,v 1.1.2.2 2003/04/26 14:37:08 rocky Exp $
+  $Id: mpeg.c,v 1.1.2.3 2003/09/14 09:32:43 rocky Exp $
 
   Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -31,7 +31,7 @@
 #include "mpeg.h"
 #include "util.h"
 
-static const char _rcsid[] = "$Id: mpeg.c,v 1.1.2.2 2003/04/26 14:37:08 rocky Exp $";
+static const char _rcsid[] = "$Id: mpeg.c,v 1.1.2.3 2003/09/14 09:32:43 rocky Exp $";
 
 #define MPEG_START_CODE_PATTERN  ((uint32_t) 0x00000100)
 #define MPEG_START_CODE_MASK     ((uint32_t) 0xffffff00)
@@ -921,7 +921,8 @@ vcd_mpeg_parse_packet (const void *_buf, unsigned buflen, bool parse_pes,
 
       vcd_warn ("mpeg scan: pack header code (0x%8.8x) expected, "
                 "but 0x%8.8x found (buflen = %d)",
-                MPEG_PACK_HEADER_CODE, _code, buflen);
+                (unsigned int) MPEG_PACK_HEADER_CODE, 
+                (unsigned int) _code, buflen);
 
       ctx->stream.packets--;
 
