@@ -1,5 +1,5 @@
 /*
-    $Id: vcd.c,v 1.1.2.9 2003/04/26 14:37:08 rocky Exp $
+    $Id: vcd.c,v 1.1.2.16 2003/09/14 09:32:43 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -29,11 +29,12 @@
 #include <math.h>
 
 #include <cdio/cdio.h>
+#include <cdio/iso9660.h>
 
 /* public headers */
 #include <libvcd/types.h>
+#include <libvcd/info.h>
 
-#include <libvcd/iso9660.h>
 #include <libvcd/files.h>
 #include <libvcd/sector.h>
 #include <libvcd/logging.h>
@@ -48,7 +49,7 @@
 #include "util.h"
 #include "vcd.h"
 
-static const char _rcsid[] = "$Id: vcd.c,v 1.1.2.9 2003/04/26 14:37:08 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd.c,v 1.1.2.16 2003/09/14 09:32:43 rocky Exp $";
 
 static const char zero[CDIO_CD_FRAMESIZE_RAW] = { 0, };
 
@@ -974,7 +975,7 @@ vcd_obj_add_dir (VcdObj *obj, const char iso_pathname[])
 
   _iso_pathname = _vcd_strdup_upper (iso_pathname);
     
-  if (!_vcd_iso_dirname_valid_p (_iso_pathname))
+  if (!iso9660_dirname_valid_p (_iso_pathname))
     {
       vcd_error("pathname `%s' is not a valid iso pathname", 
                 _iso_pathname);
@@ -1031,7 +1032,7 @@ vcd_obj_add_file (VcdObj *obj, const char iso_pathname[],
     custom_file_t *p;
     char *_iso_pathname = _vcd_strdup_upper (iso_pathname);
     
-    if (!_vcd_iso_pathname_valid_p (_iso_pathname))
+    if (!iso9660_pathname_valid_p (_iso_pathname))
       {
         vcd_error("pathname `%s' is not a valid iso pathname", 
                   _iso_pathname);
@@ -1122,12 +1123,14 @@ _finalize_vcd_iso_track_allocation (VcdObj *obj)
     {
       mpeg_segment_t *_segment = _vcd_list_node_data (node);
       
-      _segment->start_extent = _vcd_salloc (obj->iso_bitmap, SECTOR_NIL, 
-                                            _segment->segment_count * 150);
+      _segment->start_extent = 
+        _vcd_salloc (obj->iso_bitmap, SECTOR_NIL, 
+                     _segment->segment_count * VCDINFO_SEGMENT_SECTOR_SIZE);
 
       vcd_assert (_segment->start_extent % 75 == 0);
       vcd_assert (_vcd_salloc_get_highest (obj->iso_bitmap) + 1 
-              == _segment->start_extent + _segment->segment_count * 150);
+                  == _segment->start_extent 
+                  + _segment->segment_count * VCDINFO_SEGMENT_SECTOR_SIZE);
     }
 
   obj->ext_file_start_extent = _vcd_salloc_get_highest (obj->iso_bitmap) + 1;
@@ -1180,8 +1183,9 @@ _finalize_vcd_iso_track_allocation (VcdObj *obj)
   obj->iso_size =
     MAX (MIN_ISO_SIZE, _vcd_salloc_get_highest (obj->iso_bitmap) + 1);
 
-  vcd_debug ("iso9660: highest alloced sector is %d (using %d as isosize)", 
-             _vcd_salloc_get_highest (obj->iso_bitmap), obj->iso_size);
+  vcd_debug ("iso9660: highest alloced sector is %lu (using %d as isosize)", 
+             (unsigned long int) _vcd_salloc_get_highest (obj->iso_bitmap), 
+             obj->iso_size);
 
   /* after this point the ISO9660's size is frozen */
 }
@@ -1811,7 +1815,9 @@ _write_segment (VcdObj *obj, mpeg_segment_t *_segment)
 
   pause_node = _vcd_list_begin (_segment->pause_list);
 
-  for (packet_no = 0;packet_no < (_segment->segment_count * 150);packet_no++)
+  for (packet_no = 0;
+       packet_no < (_segment->segment_count * VCDINFO_SEGMENT_SECTOR_SIZE);
+       packet_no++)
     {
       uint8_t buf[M2F2_SECTOR_SIZE] = { 0, };
       uint8_t fn, cn, sm, ci;
@@ -2006,7 +2012,7 @@ _update_entry_points (VcdObj *obj)
 }
 
 static int
-_write_vcd_iso_track (VcdObj *obj)
+_write_vcd_iso_track (VcdObj *obj, const time_t *create_time)
 {
   VcdListNode *node;
   int n;
@@ -2022,18 +2028,20 @@ _write_vcd_iso_track (VcdObj *obj)
                                   _dict_get_bykey (obj, "ptm")->buf);
       
   /* generate PVD and EVD at last... */
-  set_iso_pvd (_dict_get_bykey (obj, "pvd")->buf,
-               obj->iso_volume_label, 
-               obj->iso_publisher_id,
-               obj->iso_preparer_id,
-               obj->iso_application_id, 
-               obj->iso_size, 
-               _dict_get_bykey (obj, "dir")->buf, 
-               _dict_get_bykey (obj, "ptl")->sector,
-               _dict_get_bykey (obj, "ptm")->sector,
-               pathtable_get_size (_dict_get_bykey (obj, "ptm")->buf));
+  iso9660_set_pvd (_dict_get_bykey (obj, "pvd")->buf,
+                   obj->iso_volume_label, 
+                   obj->iso_publisher_id,
+                   obj->iso_preparer_id,
+                   obj->iso_application_id, 
+                   obj->iso_size, 
+                   _dict_get_bykey (obj, "dir")->buf, 
+                   _dict_get_bykey (obj, "ptl")->sector,
+                   _dict_get_bykey (obj, "ptm")->sector,
+                   iso9660_pathtable_get_size (_dict_get_bykey (obj, "ptm")->buf),
+                   create_time
+);
     
-  set_iso_evd (_dict_get_bykey (obj, "evd")->buf);
+  iso9660_set_evd (_dict_get_bykey (obj, "evd")->buf);
 
   /* fill VCD relevant files with data */
 
@@ -2190,7 +2198,7 @@ vcd_obj_begin_output (VcdObj *obj)
                (unsigned) image_size, (unsigned) CDIO_CD_MAX_SECTORS);
 
   {
-    char *_tmp = _vcd_lba_to_msf_str (image_size);
+    char *_tmp = cdio_lba_to_msf_str (image_size);
 
     if (image_size > CDIO_CD_74MIN_SECTORS)
       vcd_warn ("generated image (%d sectors [%s]) may not fit "
@@ -2244,7 +2252,8 @@ vcd_obj_append_pbc_node (VcdObj *obj, struct _pbc_t *_pbc)
 
 int
 vcd_obj_write_image (VcdObj *obj, VcdImageSink *image_sink,
-                     progress_callback_t callback, void *user_data)
+                     progress_callback_t callback, void *user_data,
+                     const time_t *create_time)
 {
   VcdListNode *node;
 
@@ -2332,7 +2341,7 @@ vcd_obj_write_image (VcdObj *obj, VcdImageSink *image_sink,
     if (_callback_wrapper (obj, true))
       return 1;
 
-    if (_write_vcd_iso_track (obj))
+    if (_write_vcd_iso_track (obj, create_time))
       return 1;
 
     if (obj->update_scan_offsets)
